Index: apps/app_voicemail.c
===================================================================
--- apps/app_voicemail.c	(revision 7681)
+++ apps/app_voicemail.c	(working copy)
@@ -121,11 +121,13 @@
 	OPT_RECORDGAIN =       (1 << 3),
 	OPT_PREPEND_MAILBOX =  (1 << 4),
 	OPT_PRIORITY_JUMP =    (1 << 5),
+	OPT_AUTOPLAY =         (1 << 6),
 } vm_option_flags;
 
 enum {
 	OPT_ARG_RECORDGAIN = 0,
 	OPT_ARG_ARRAY_SIZE = 1,
+	OPT_ARG_PLAYFOLDER = 2,
 } vm_option_args;
 
 AST_APP_OPTIONS(vm_app_options, {
@@ -135,6 +137,7 @@
 	AST_APP_OPTION_ARG('g', OPT_RECORDGAIN, OPT_ARG_RECORDGAIN),
 	AST_APP_OPTION('p', OPT_PREPEND_MAILBOX),
 	AST_APP_OPTION('j', OPT_PRIORITY_JUMP),
+	AST_APP_OPTION_ARG('a', OPT_AUTOPLAY, OPT_ARG_PLAYFOLDER),
 });
 
 static int load_config(void);
@@ -5017,6 +5022,8 @@
 	int silentexit = 0;
 	struct ast_flags flags = { 0 };
 	signed char record_gain = 0;
+	int play_auto = 0;
+	int play_folder = 0;
 
 	LOCAL_USER_ADD(u);
 
@@ -5052,6 +5059,14 @@
 					record_gain = (signed char) gain;
 				}
 			}
+			if (ast_test_flag(&flags, OPT_AUTOPLAY)) {
+				play_auto = 1;
+				if (sscanf(opts[OPT_ARG_PLAYFOLDER], "%d", &play_folder) != 1) {
+					ast_log(LOG_WARNING, "Invalid value '%s' provided for folder autoplay option\n", opts[OPT_ARG_PLAYFOLDER]);
+					LOCAL_USER_REMOVE(u);
+					return -1;
+				}
+			}
 		} else {
 			/* old style options parsing */
 			while (*argv[0]) {
@@ -5122,11 +5137,24 @@
 	vms.newmessages = vms.lastmsg + 1;
 		
 	/* Select proper mailbox FIRST!! */
-	if (!vms.newmessages && vms.oldmessages) {
-		/* If we only have old messages start here */
-		res = open_mailbox(&vms, vmu, 1);
+	if (play_auto) {
+		res = open_mailbox(&vms, vmu, play_folder);
 		if (res == ERROR_LOCK_PATH)
 			goto out;
+
+		/* If there are no new messages, inform the user and hangup */
+		if (vms.lastmsg == -1) {
+			cmd = vm_browse_messages(chan, &vms, vmu);
+			res = 0;
+			goto out;
+		}
+	} else {
+		if (!vms.newmessages && vms.oldmessages) {
+			/* If we only have old messages start here */
+			res = open_mailbox(&vms, vmu, 1);
+			if (res == ERROR_LOCK_PATH)
+				goto out;
+		}
 	}
 
 	if (useadsi)
@@ -5150,7 +5178,11 @@
 		}
 	}
 
-	cmd = vm_intro(chan, &vms);
+	if (play_auto) {
+		cmd = '1';
+	} else {
+		cmd = vm_intro(chan, &vms);
+	}
 
 	vms.repeats = 0;
 	vms.starting = 1;
